-- OPEN.ADA   Ver. 4.01   2001-SEP-10   Copyright 1988-2001 John J. Herro
--
-- SOFTWARE INNOVATIONS TECHNOLOGY          www.adatutor.com
-- 1083 MANDARIN DR NE
-- PALM BAY FL 32905-4706                   john@adatutor.com
--
-- (321) 951-0233
--
-- Compile this before compiling ADATU401.ADA, when using a PC with an Open
-- Ada compiler, now sold by D.C. Heath Co.
--
with Tty;
package Custom_IO is
  type Color is (Black, Red, Green, Yellow, Blue, Magenta, Cyan, White);
  Foregrnd_Color   : Color;
  Backgrnd_Color   : Color;
  Border_Color     : Color;
  Fore_Color_Digit : Character;
  Back_Color_Digit : Character;
  Esc              : constant Character := Character'Val(27);
  Normal_Colors    : String(1 .. 10) := Esc & "[0;37;40m";
  Clear_Scrn       : constant String := Esc & "[H" & Esc &"[2J";

  procedure Set_Border_Color (To   : in  Color);
  procedure Get              (Char : out Character);
  procedure Put              (Char : in  Character) renames Tty.Put;
  procedure Put              (Str  : in  String)    renames Tty.Put;
  procedure Put_Line         (Str  : in  String)    renames Tty.Put_Line;
  procedure Get_Line         (Str  : out String; Last : out Natural);
  procedure New_Line;
end Custom_IO;

with Interrupt;
package body Custom_IO is
  procedure Set_Border_Color(To : in Color) is
      --
      -- This procedure sets the border color on a PC by calling interrupt
      -- 10 hex.  Before the call, register AH is set to service number 0B hex,
      -- BH is set to zero, and BL is set to an integer as shown in the
      -- declaration of Color_Number below.  Note that the integers in
      -- Color_Number are bit reversed from the integers defining foreground
      -- and background colors in ANSI escape sequences.  Note also that some
      -- color PCs don't have separate border colors.
      --
    Regis_Block  : Interrupt.Registers;
    Color_Number : constant array(Color) of Integer :=
        (Black   => 0,   Red     => 4,   Green   => 2,   Yellow  => 6,
         Blue    => 1,   Magenta => 5,   Cyan    => 3,   White   => 7);
  begin
    Regis_Block.AX := 16#0B00#;
    Regis_Block.BX := 16#0000# + Color_Number(To);
    Interrupt.Vector(On => 16#10#, Register_Block => Regis_Block);
  end Set_Border_Color;

  procedure Get(Char : out Character) is
  begin
    Char := Tty.Get(No_Echo => True);
  end Get;

  procedure Get_Line(Str : out String; Last : out Natural) is
  begin
    Tty.Get(Str, Last);
    New_Line;
  end Get_Line;

  procedure New_Line is
  begin
    Put_Line("");
  end New_Line;
end Custom_IO;
