-- VAX.ADA   Ver. 4.01   2001-SEP-10   Copyright 1988-2001 John J. Herro
--
-- SOFTWARE INNOVATIONS TECHNOLOGY          www.adatutor.com
-- 1083 MANDARIN DR NE
-- PALM BAY FL 32905-4706                   john@adatutor.com
--
-- (321) 951-0233
--
-- Compile this before compiling ADATU401.ADA with VAX Ada.  See the first page
-- of ADATU401.ADA for more details.
--
-- This code is written in VAX Ada 83 and will compile on VAX Ada 83 and
-- VAX Ada 95 compilers.  If you have an Ada 95 compiler, we recommend
-- compiling ADA95.ADA instead of VAX.ADA.
--
package Custom_IO is
  type Color is (Black, Red, Green, Yellow, Blue, Magenta, Cyan, White);
  Foregrnd_Color   : Color;
  Backgrnd_Color   : Color;
  Border_Color     : Color;
  Fore_Color_Digit : Character;
  Back_Color_Digit : Character;
  Esc              : constant Character := Character'Val(27);
  Normal_Colors    : String(1 .. 10) := Esc & "[0;37;40m";
  Clear_Scrn       : constant String := Esc & "[H" & Esc & "[2J";

  procedure Set_Border_Color (To   : in  Color);
  procedure Get              (Char : out Character);
  procedure Put              (Char : in  Character);
  procedure Put              (Str  : in  String);
  procedure Put_Line         (Str  : in  String);
  procedure Get_Line         (Str  : out String; Last : out Natural);
  procedure New_Line;
end Custom_IO;

with Starlet, System; use Starlet, System;
package body Custom_IO is
  Beep : constant Character := Character'Val(7);
  BS   : constant Character := Character'Val(8);
  LF   : constant Character := Character'Val(10);
  CR   : constant Character := Character'Val(13);
  Del  : constant Character := Character'Val(127);
  Chan : Starlet.Channel_Type;
  IOSB : System.Unsigned_Quadword;
  Stat : System.Unsigned_Longword;
  procedure QIOW(Stat   : out Unsigned_Longword;
                 EFN    : in Integer;
                 Chan   : in Channel_Type;
                 Func   : in Short_Integer;
                 IOSB   : out Unsigned_Quadword;
                 ASTadr : in Integer;
                 ASTPRM : in Integer;
                 P1     : in out String;
                 P2, P3 : in Integer;
                 P4     : in Unsigned_Quadword;
                 P5, P6 : in Integer);
  pragma Interface(System_Library, QIOW);
  pragma Import_Valued_Procedure(
              Internal => QIOW,
              External => "SYS$QIOW",
              Parameter_Types => (
                   Unsigned_Longword,
                   Integer,
                   Channel_Type,
                   Short_Integer,
                   Unsigned_Quadword,
                   Integer,
                   Integer,
                   String,
                   Integer,
                   Integer,
                   Unsigned_Quadword,
                   Integer,
                   Integer),
              Mechanism => (
                   Value,
                   Value,
                   Value,
                   Value,
                   Reference,
                   Value,
                   Reference,
                   Reference,
                   Value,
                   Reference,
                   Reference,
                   Reference,
                   Reference));

  procedure Set_Border_Color(To : in Color) is
     -- Dummy procedure for computers other than PCs.
  begin
    null;
  end Set_Border_Color;

  procedure Get(Char : out Character) is
    S : String(1 .. 1);
  begin
    QIOW(Stat   => Stat,
         EFN    => 0,
         Chan   => Chan,
         Func   => 16#7A#,
         IOSB   => IOSB,
         ASTadr => 0,
         ASTPRM => 0,
         P1     => S,
         P2     => 1,
         P3     => 0,
         P4     => (0,0),
         P5     => 0,
         P6     => 0);
    Char := S(1);
  end Get;

  procedure Put(Char : in Character) is
  begin
    Put(Char & "");
  end Put;

  procedure Put(Str : in String) is
    S : String(Str'Range) := Str;
  begin
    QIOW(Stat   => Stat,
         EFN    => 0,
         Chan   => Chan,
         Func   => 16#70#,
         IOSB   => IOSB,
         ASTadr => 0,
         ASTPRM => 0,
         P1     => S,
         P2     => S'Length,
         P3     => 0,
         P4     => (0,0),
         P5     => 0,
         P6     => 0);
   end Put;

  procedure Put_Line(Str : in String) is
  begin
    Put(Str & CR & LF);
  end Put_Line;

  procedure Get_Line(Str : out String; Last : out Natural) is separate;

  procedure New_Line is
  begin
    Put(CR & LF);
  end New_Line;
begin
  Starlet.Assign(Stat, "TT:", Chan);
end Custom_IO;

-- This procedure gets a string from the terminal, while allowing typing errors
-- to be corrected.
--
separate (Custom_IO)
procedure Get_Line(Str : out String; Last : out Natural) is
  S     : String(Str'Range);                              -- Local copy of Str.
  Char  : Character := ' ';                     -- One character from keyboard.
  Place : Integer   := Str'First;      -- Position of next available character.
begin
  while Char /= CR loop                          -- CR signifies end of string.
    Get(Char);                                            -- Get one character.
    if Char = CR then
      New_Line;                          -- Give new line at end of the string.
    elsif Char = BS or Char = Del then
      if Place > Str'First then           -- Ignore BS/DEL when string is null.
        Put(BS & ' ' & BS);                   -- Erase last char. from display.
        Place := Place - 1;                   -- Remove last char. from string.
      end if;
    elsif Place > Str'Last then      -- Beep when length of string is exceeded.
      Put(Beep);
    else
      Put(Char);                                   -- Echo the character typed.
      S(Place) := Char;                         -- Add character to the string.
      Place := Place + 1;
    end if;
  end loop;
  Str(Str'First .. Place - 1) := S(Str'First .. Place - 1);
  Last := Place - 1;
end Get_Line;
